@@ -31,10 +31,11 @@ import metator.log as mtl
 import metator.network as mtn
 import metator.partition as mtp
 import metator.validation as mtv
+import metator.contact_map as mtc
 from docopt import docopt
 from metator.log import logger
 from os.path import exists, dirname, join
-from scipy.sparse import save_npz
+from scipy.sparse import save_npz, load_npz
 
 
 class AbstractCommand:
@@ -83,10 +84,11 @@ class Network(AbstractCommand):
     every single node in the network is written on a 'contig data' file.
 
     usage:
-        network --forward=STR --assembly=FILE [--reverse=STR] [--depth=FILE]
+        network --forward=STR --assembly=FILE [--reverse=STR]
+        [--aligner=bowtie2] [--aligner-mode=normal] [--depth=FILE]
         [--enzyme=STR] [--normalization=empirical_hit] [--no-clean-up]
         [--outdir=DIR] [--min-quality=30] [--self-contacts] [--start=fastq]
-        [--threads=1] [--tempdir=DIR]
+        [--threads=1] [--tmpdir=DIR]
 
     options:
         -1, --forward=STR       Fastq file or list of Fastq separated by a comma
@@ -101,6 +103,10 @@ class Network(AbstractCommand):
         -a, --assembly=FILE     The initial assembly path acting as the
                                 alignment file's reference genome or the
                                 basename of the bowtie2 index.
+        -b, --aligner=STR       Aligner algorithm to use. Either "bwa" or
+                                "bowtie2". [Default: bowtie2]
+        -B, --aligner-mode=STR  Mode of alignment from hicstuff. Either normal,
+                                iterative or cutsite. [Default: normal]
         -d, --depth=FILE        The depth.txt file from the shotgun reads used
                                 to made the assembly computed by
                                 jgi_summarize_bam_contig_depths from metabat2
@@ -126,16 +132,18 @@ class Network(AbstractCommand):
                                 "bam", or "pair". [Default: fastq]
         -t, --threads=INT       Number of parallel threads allocated for the
                                 alignement. [Default: 1]
-        -T, --tempdir=DIR       Temporary directory. Default to current
+        -T, --tmpdir=DIR       Temporary directory. Default to current
                                 directory. [Default: ./tmp]
     """
 
     def execute(self):
 
         # Defined the temporary directory.
-        if not self.args["--tempdir"]:
-            self.args["--tempdir"] = "./tmp"
-        temp_directory = mio.generate_temp_dir(self.args["--tempdir"])
+        if not self.args["--tmpdir"]:
+            tmp_dir = mio.generate_temp_dir("./tmp")
+        else:
+            tmp_dir = self.args["--tmpdir"]
+            os.makedirs(tmp_dir, exist_ok=True)
 
         # Defined the output directory and output file names.
         if not self.args["--outdir"]:
@@ -156,12 +164,43 @@ class Network(AbstractCommand):
         self_contacts = self.args["--self-contacts"]
 
         # Check if forward and reverse arguments are given:
-        if self.args["--start"] != "pair":
-            if not self.args["--forward"] or not self.args["--reverse"]:
-                logger.error(
-                    "Forward and reverse arguments are necessary for fastq or bam start."
-                )
-                raise ValueError
+        if (
+            self.args["--start"] == "fastq"
+            or (
+                self.args["--start"] == "bam"
+                and self.args["--aligner"] == "bowtie2"
+            )
+        ) and not self.args["--reverse"]:
+            logger.error(
+                "Forward and reverse arguments are necessary for fastq with %s start and %s aligner.",
+                self.args["--start"],
+                self.args["--aligner"],
+            )
+            raise ValueError
+
+        # Check aligner.
+        if self.args["--aligner"] not in ["bowtie2", "bwa"]:
+            logger.error('Aligner should be either "bowtie2" or "bwa".')
+
+        # Check aligner mode.
+        if self.args["--aligner-mode"] not in [
+            "normal",
+            "iterative",
+            "cutsiste",
+        ]:
+            logger.error(
+                'Aligner mode should be either "normal", "iterative" or "cutsite".'
+            )
+        if (
+            self.args["--aligner-mode"] == "cutsite"
+            and not self.args["--enzyme"]
+        ):
+            logger.warning(
+                "cutsite mode required an enzyme. Iterative mode will be use instead."
+            )
+            self.args["--aligner-mode"] = "iterative"
+
+        # Check correct algorithm value.
 
         # Check if normalization in the list of possible normalization.
         list_normalization = [
@@ -195,32 +234,38 @@ class Network(AbstractCommand):
                 'For "abundance" and "theoritical_hit" normalization, depth is required.'
             )
             raise ValueError
-
+        if self.args["--start"] not in ["fastq", "bam", "pair", "network"]:
+            logger.error(
+                "Start argument should be 'fastq', 'bam', 'pair' or 'network'."
+            )
+            raise ValueError
         # Extract index and genome file
         assembly = self.args["--assembly"]
         # Check what is the reference. If a fasta is given build the index. If a
         # bowtie2 index is given, retreive the fasta.
-        index = mio.check_fasta_index(assembly, mode="bowtie2")
+        index = mio.check_fasta_index(assembly, mode=self.args["--aligner"])
         if index is None:
             if mio.check_is_fasta(assembly):
                 fasta = assembly
                 # If start at bam could skip the index generation.
                 if self.args["--start"] == "fastq":
-                    index = mio.generate_fasta_index(fasta, temp_directory)
+                    index = mio.generate_fasta_index(
+                        fasta, self.args["--aligner"], tmp_dir
+                    )
             else:
                 logger.error(
-                    "Please give as assembly argument a bowtie2 index or a fasta."
+                    "Please give as assembly argument a %s index or a fasta.",
+                    self.args["--aligner"],
                 )
                 raise ValueError
         else:
-            fasta = mio.retrieve_fasta(index, temp_directory)
+            fasta = mio.retrieve_fasta(index, self.args["--aligner"], tmp_dir)
 
         # Print information of teh workflow:
-        logger.info("Enzyme: {0}".format(self.args["--enzyme"]))
-        logger.info("Normalization: {0}".format(self.args["--normalization"]))
-        logger.info(
-            "Minimum mapping quality: {0}".format(self.args["--min-quality"])
-        )
+        logger.info("Aligner algorithm: %s", self.args["--aligner"])
+        logger.info("Enzyme: %s", self.args["--enzyme"])
+        logger.info("Normalization: %s", self.args["--normalization"])
+        logger.info("Minimum mapping quality: %s", self.args["--min-quality"])
 
         # Do not align if pair start
         if self.args["--start"] == "pair":
@@ -240,12 +285,14 @@ class Network(AbstractCommand):
                 self.args["--reverse"],
                 index,
                 fasta,
+                self.args["--aligner"],
+                self.args["--aligner-mode"],
                 min_qual,
                 self.args["--start"],
                 self.args["--depth"],
                 self.args["--enzyme"],
                 self.args["--outdir"],
-                temp_directory,
+                tmp_dir,
                 self.args["--threads"],
             )
 
@@ -257,7 +304,7 @@ class Network(AbstractCommand):
             self.args["--outdir"],
             "network.txt",
             "contig_data_network.txt",
-            temp_directory,
+            tmp_dir,
             self.args["--threads"],
             self.args["--normalization"],
             self_contacts,
@@ -265,7 +312,7 @@ class Network(AbstractCommand):
 
         # Delete the temporary folder
         if not self.args["--no-clean-up"]:
-            shutil.rmtree(temp_directory)
+            shutil.rmtree(tmp_dir)
 
 
 class Partition(AbstractCommand):
@@ -292,9 +339,9 @@ class Partition(AbstractCommand):
 
     usage:
         partition  --assembly=FILE --contigs=FILE --network=FILE
-        [--algorithm=louvain] [--force] [--iterations=100] [--no-clean-up]
-        [--outdir=DIR] [--overlap=80] [--res-param=1.0] [--size=500000]
-        [--threads=1] [--tempdir=DIR]
+        [--algorithm=louvain] [--cluster-matrix] [--force] [--iterations=100]
+        [--no-clean-up] [--outdir=DIR] [--overlap=80] [--res-param=1.0]
+        [--size=500000] [--threads=1] [--tmpdir=DIR]
 
     options:
         -a, --assembly=FILE     The path to the assembly fasta file used to do
@@ -304,6 +351,7 @@ class Partition(AbstractCommand):
         -c, --contigs=FILE      The path to the tsv file containing the data of
                                 the contigs (ID, Name, Length, GC content, Hit,
                                 Coverage, Restriction Site).
+        -C, --cluster-matrix    If enabled, save the clustering matrix.
         -F, --force             If enabled, would remove directory of
                                 overlapping bins in the output directory.
         -i, --iterations=INT    Number of iterations of Louvain. [Default: 100]
@@ -321,16 +369,18 @@ class Partition(AbstractCommand):
                                 [Default: 500000]
         -t, --threads=INT       Number of parallel threads allocated for the
                                 partition. [Default: 1]
-        -T, --tempdir=DIR       Temporary directory. Default to current
+        -T, --tmpdir=DIR       Temporary directory. Default to current
                                 directory. [Default: ./tmp]
     """
 
     def execute(self):
 
         # Defined the temporary directory.
-        if not self.args["--tempdir"]:
-            self.args["--tempdir"] = "./tmp"
-        temp_directory = mio.generate_temp_dir(self.args["--tempdir"])
+        if not self.args["--tmpdir"]:
+            tmp_dir = mio.generate_temp_dir("./tmp")
+        else:
+            tmp_dir = self.args["--tmpdir"]
+            os.makedirs(tmp_dir, exist_ok=True)
 
         # Defined the output directory.
         if not self.args["--outdir"]:
@@ -345,9 +395,8 @@ class Partition(AbstractCommand):
                 os.makedirs(fasta_dir)
             else:
                 logger.error(
-                    "{0} already existed. Remove directory or use -F argument to overwrite it.".format(
-                        fasta_dir
-                    )
+                    "%s already existed. Remove directory or use -F argument to overwrite it.",
+                    fasta_dir,
                 )
                 raise ValueError
 
@@ -371,9 +420,10 @@ class Partition(AbstractCommand):
             raise ValueError
 
         # Partition the network
-        clustering_matrix, contigs_data_file = mtp.partition(
+        _clustering_matrix_file, _contigs_data_file = mtp.partition(
             self.args["--algorithm"],
             self.args["--assembly"],
+            self.args["--cluster-matrix"],
             self.args["--contigs"],
             iterations,
             self.args["--network"],
@@ -382,19 +432,15 @@ class Partition(AbstractCommand):
             overlapping_parameter,
             resolution_parameter,
             size,
-            temp_directory,
+            tmp_dir,
             threads,
         )
 
-        # Save the clustering matrix
-        clustering_matrix_file = join(
-            self.args["--outdir"], "clustering_matrix_partition.txt"
-        )
-        save_npz(clustering_matrix_file, clustering_matrix)
-
+        # Delete pyfastx index:
+        os.remove(self.args["--assembly"] + ".fxi")
         # Delete the temporary folder
         if not self.args["--no-clean-up"]:
-            shutil.rmtree(temp_directory)
+            shutil.rmtree(tmp_dir)
 
 
 class Validation(AbstractCommand):
@@ -411,9 +457,9 @@ class Validation(AbstractCommand):
 
     usage:
         validation --assembly=FILE --contigs=FILE --fasta=DIR --network=FILE
-        [--algorithm=louvain] [--force] [--iterations=10] [--no-clean-up]
-        [--outdir=DIR] [--overlap=90] [--res-param=1.0] [--size=500000]
-        [--threads=1] [--tempdir=DIR]
+        [--algorithm=louvain] [--cluster-matrix] [--force] [--iterations=10]
+        [--no-clean-up] [--outdir=DIR] [--overlap=90] [--res-param=1.0]
+        [--size=500000] [--threads=1] [--tmpdir=DIR]
 
     options:
         -a, --assembly=FILE     The path to the assembly fasta file used to do
@@ -422,6 +468,7 @@ class Validation(AbstractCommand):
                                 [Default: louvain]
         -c, --contigs=FILE      The path to the file containing the data of the
                                 contigs from the partition step (13 columns).
+        -C, --cluster-matrix    If enabled, save the clustering matrix.
         -f, --fasta=DIR         Path to the directory containing the input fasta
                                 files of the bins to decontaminate.
         -F, --force             If enable, would remove directory of recursive
@@ -442,7 +489,7 @@ class Validation(AbstractCommand):
                                 [Default: 500000]
         -t, --threads=INT       Number of parallel threads allocated for the
                                 partition. [Default: 1]
-        -T, --tempdir=DIR       Temporary directory. Default to current
+        -T, --tmpdir=DIR       Temporary directory. Default to current
                                 directory. [Default: ./tmp]
     """
 
@@ -453,9 +500,11 @@ class Validation(AbstractCommand):
     def execute(self):
 
         # Defined the temporary directory.
-        if not self.args["--tempdir"]:
-            self.args["--tempdir"] = "./tmp"
-        temp_directory = mio.generate_temp_dir(self.args["--tempdir"])
+        if not self.args["--tmpdir"]:
+            tmp_dir = mio.generate_temp_dir("./tmp")
+        else:
+            tmp_dir = self.args["--tmpdir"]
+            os.makedirs(tmp_dir, exist_ok=True)
 
         # Defined the output directory and output file names.
         if not self.args["--outdir"]:
@@ -471,9 +520,8 @@ class Validation(AbstractCommand):
                 os.makedirs(recursive_fasta_dir)
             else:
                 logger.error(
-                    "{0} already existed. Remove directory or use -F argument to overwrite it.".format(
-                        recursive_fasta_dir
-                    )
+                    "%s already existed. Remove directory or use -F argument to overwrite it.",
+                    recursive_fasta_dir,
                 )
                 raise ValueError
         if not exists(final_fasta_dir):
@@ -484,9 +532,8 @@ class Validation(AbstractCommand):
                 os.makedirs(final_fasta_dir)
             else:
                 logger.error(
-                    "{0} already existed. Remove directory or use -F argument to overwrite it.".format(
-                        final_fasta_dir
-                    )
+                    "%s already existed. Remove directory or use -F argument to overwrite it.",
+                    final_fasta_dir,
                 )
                 raise ValueError
 
@@ -504,7 +551,7 @@ class Validation(AbstractCommand):
         overlapping_parameter = int(self.args["--overlap"]) / 100
         resolution_parameter = float(self.args["--res-param"])
 
-        # Check checkM availability
+        # Check checkM availabilitys
         if not mio.check_checkm():
             logger.error(
                 "CheckM is not in the path. Could not make the iterations"
@@ -516,9 +563,10 @@ class Validation(AbstractCommand):
             logger.error('algorithm should be either "louvain" or "leiden"')
             raise ValueError
 
-        clustering_matrix = mtv.recursive_decontamination(
+        _clustering_matrix_file = mtv.recursive_decontamination(
             self.args["--algorithm"],
             self.args["--assembly"],
+            self.args["--cluster-matrix"],
             self.args["--contigs"],
             final_fasta_dir,
             self.args["--fasta"],
@@ -529,19 +577,15 @@ class Validation(AbstractCommand):
             recursive_fasta_dir,
             resolution_parameter,
             size,
-            temp_directory,
+            tmp_dir,
             threads,
         )
 
-        # Save the clustering matrix
-        clustering_matrix_file = join(
-            self.args["--outdir"], "clustering_matrix_recursive.txt"
-        )
-        save_npz(clustering_matrix_file, clustering_matrix)
-
+        # Delete pyfastx index:
+        os.remove(self.args["--assembly"] + ".fxi")
         # Delete the temporary folder
         if not self.args["--no-clean-up"]:
-            shutil.rmtree(temp_directory)
+            shutil.rmtree(tmp_dir)
 
 
 class Pipeline(AbstractCommand):
@@ -556,11 +600,12 @@ class Pipeline(AbstractCommand):
 
     usage:
         pipeline --assembly=FILE [--forward=STR] [--reverse=STR]
-        [--algorithm=louvain] [--contigs=FILE] [--depth=FILE] [--enzyme=STR]
+        [--algorithm=louvain] [--aligner=bowtie2] [--aligner-mode=normal]
+        [--cluster-matrix] [--contigs=FILE] [--depth=FILE] [--enzyme=STR]
         [--force] [--iterations=100] [--rec-iter=10] [--network=FILE]
         [--no-clean-up] [--normalization=empirical_hit] [--outdir=DIR]
         [--overlap=80] [--rec-overlap=90]  [--min-quality=30] [--res-param=1.0]
-        [--size=500000] [--start=fastq] [--threads=1] [--tempdir=DIR]
+        [--size=500000] [--start=fastq] [--threads=1] [--tmpdir=DIR]
         [--skip-validation]
 
     options:
@@ -578,9 +623,15 @@ class Pipeline(AbstractCommand):
                                 basename of the bowtie2 index.
         -A, --algorithm=STR     Algorithm to use. Either "louvain" or "leiden".
                                 [Default: louvain]
+        -b, --aligner=STR       Aligner algorithm to use. Either "bwa" or
+                                "bowtie2". [Default: bowtie2]
+        -B, --aligner-mode=STR  Mode of alignment from hicstuff. Either normal,
+                                iterative or cutsite. [Default: normal]
         -c, --contigs=FILE      The path to the file containing the data ofthe
                                 contigs (ID, Name, Length, GC content, Hit,
-                                Coverage, Restriction site).
+                                Coverage, Restriction site). Required if start
+                                network.
+        -C, --cluster-matrix    If enabled, save the clustering matrix.
         -d, --depth=FILE        The depth.txt file from the shotgun reads used
                                 to made the assembly computed by
                                 jgi_summarize_bam_contig_depths from metabat2
@@ -622,7 +673,7 @@ class Pipeline(AbstractCommand):
                                 pair, or network. [Default: fastq]
         -t, --threads=INT       Number of parallel threads allocated for the
                                 alignement. [Default: 1]
-        -T, --tempdir=DIR       Temporary directory. Default to current
+        -T, --tmpdir=DIR       Temporary directory. Default to current
                                 directory. [Default: ./tmp]
         -v, --skip-validation   If  enables do not do the validation step which
                                 have an high memory usage (checkM ~ 40G)
@@ -631,9 +682,11 @@ class Pipeline(AbstractCommand):
     def execute(self):
 
         # Defined the temporary directory.
-        if not self.args["--tempdir"]:
-            self.args["--tempdir"] = "./tmp"
-        temp_directory = mio.generate_temp_dir(self.args["--tempdir"])
+        if not self.args["--tmpdir"]:
+            tmp_dir = mio.generate_temp_dir("./tmp")
+        else:
+            tmp_dir = self.args["--tmpdir"]
+            os.makedirs(tmp_dir, exist_ok=True)
 
         # Defined the output directory and output file names.
         if not self.args["--outdir"]:
@@ -649,9 +702,8 @@ class Pipeline(AbstractCommand):
             else:
                 print(self.args["--force"])
                 logger.error(
-                    "{0} already existed. Remove directory or use -F argument to overwrite it.".format(
-                        overlapping_fasta_dir
-                    )
+                    "%s already existed. Remove directory or use -F argument to overwrite it.",
+                    overlapping_fasta_dir,
                 )
                 raise ValueError
 
@@ -670,7 +722,29 @@ class Pipeline(AbstractCommand):
         threads = int(self.args["--threads"])
         resolution_parameter = float(self.args["--res-param"])
 
-        # Check correct algorithm value
+        # Check aligner.
+        if self.args["--aligner"] not in ["bowtie2", "bwa"]:
+            logger.error('Aligner should be either "bowtie2" or "bwa".')
+
+        # Check aligner mode.
+        if self.args["--aligner-mode"] not in [
+            "normal",
+            "iterative",
+            "cutsiste",
+        ]:
+            logger.error(
+                'Aligner mode should be either "normal", "iterative" or "cutsite".'
+            )
+        if (
+            self.args["--aligner-mode"] == "cutsite"
+            and not self.args["--enzyme"]
+        ):
+            logger.warning(
+                "cutsite mode required an enzyme. Iterative mode will be use instead."
+            )
+            self.args["--aligner-mode"] = "iterative"
+
+        # Check correct algorithm value.
         if self.args["--algorithm"] not in ["louvain", "leiden"]:
             logger.error('algorithm should be either "louvain" or "leiden"')
             raise ValueError
@@ -719,9 +793,8 @@ class Pipeline(AbstractCommand):
                     os.makedirs(recursive_fasta_dir)
                 else:
                     logger.error(
-                        "{0} already existed. Remove directory or use -F argument to overwrite it.".format(
-                            recursive_fasta_dir
-                        )
+                        "%s already existed. Remove directory or use -F argument to overwrite it",
+                        recursive_fasta_dir,
                     )
                     raise ValueError
             final_fasta_dir = join(self.args["--outdir"], "final_bin")
@@ -733,9 +806,8 @@ class Pipeline(AbstractCommand):
                     os.makedirs(final_fasta_dir)
                 else:
                     logger.error(
-                        "{0} already existed. Remove directory or use -F argument to overwrite it.".format(
-                            final_fasta_dir
-                        )
+                        "%s already existed. Remove directory or use -F argument to overwrite it.",
+                        final_fasta_dir,
                     )
                     raise ValueError
 
@@ -762,53 +834,58 @@ class Pipeline(AbstractCommand):
             raise ValueError
 
         # Check if forward and reverse reads are given for fastq and bam start.
-        if start <= 2:
-            if not self.args["--forward"] or not self.args["--reverse"]:
-                logger.error(
-                    "Forward and reverse arguments are necessary for fastq or bam start."
-                )
-                raise ValueError
+        if (
+            self.args["--start"] == "fastq"
+            or (
+                self.args["--start"] == "bam"
+                and self.args["--aligner"] == "bowtie2"
+            )
+        ) and not self.args["--reverse"]:
+            logger.error(
+                "Forward and reverse arguments are necessary for fastq with %s start and %s aligner.",
+                self.args["--start"],
+                self.args["--aligner"],
+            )
+            raise ValueError
 
         # Print information of the workflow:
         if start == 1:
-            logger.info("Minimum mapping quality: {0}".format(min_qual))
+            logger.info("Minimum mapping quality: %d", min_qual)
         if start <= 2:
-            logger.info("Enzyme: {0}".format(self.args["--enzyme"]))
-            logger.info(
-                "Normalization: {0}".format(self.args["--normalization"])
-            )
-        logger.info("Partition algorithm: {0}".format(self.args["--algorithm"]))
-        logger.info("Partition iterations: {0}".format(iterations))
-        logger.info("Overlapping parameter: {0}".format(overlapping_parameter))
+            logger.info("Enzyme: %s", self.args["--enzyme"])
+            logger.info("Normalization: %s", self.args["--normalization"])
+        logger.info("Aligner algorithm: %s", self.args["--aligner"])
+        logger.info("Partition algorithm: %s", self.args["--algorithm"])
+        logger.info("Partition iterations: %s", iterations)
+        logger.info("Overlapping parameter: %s", overlapping_parameter)
         if not self.args["--skip-validation"]:
             logger.info(
-                "Recursive partition iterations: {0}".format(
-                    recursive_iterations
-                )
+                "Recursive partition iterations: %d", recursive_iterations
             )
             logger.info(
-                "Recursive overlapping parameter: {0}".format(
-                    recursive_overlapping_parameter
-                )
+                "Recursive overlapping parameter: %s",
+                recursive_overlapping_parameter,
             )
 
         # Extract index and genome file
         assembly = self.args["--assembly"]
         # Check what is the reference. If a fasta is given build the index. If a
         # bowtie2 index is given, retreive the fasta.
-        index = mio.check_fasta_index(assembly, mode="bowtie2")
+        index = mio.check_fasta_index(assembly, mode=self.args["--aligner"])
         if index is None:
             if mio.check_is_fasta(assembly):
                 fasta = assembly
                 if start == 1:
-                    index = mio.generate_fasta_index(fasta, temp_directory)
+                    index = mio.generate_fasta_index(
+                        fasta, self.args["--aligner"], tmp_dir
+                    )
             else:
                 logger.error(
                     "Please give as assembly argument a bowtie2 index or a fasta."
                 )
                 raise ValueError
         else:
-            fasta = mio.retrieve_fasta(index, temp_directory)
+            fasta = mio.retrieve_fasta(index, self.args["--aligner"], tmp_dir)
 
         # Run the whole workflow
         if start <= 3:
@@ -819,12 +896,14 @@ class Pipeline(AbstractCommand):
                     self.args["--reverse"],
                     index,
                     fasta,
+                    self.args["--aligner"],
+                    self.args["--aligner-mode"],
                     min_qual,
                     self.args["--start"],
                     self.args["--depth"],
                     self.args["--enzyme"],
                     self.args["--outdir"],
-                    temp_directory,
+                    tmp_dir,
                     self.args["--threads"],
                 )
             else:
@@ -844,7 +923,7 @@ class Pipeline(AbstractCommand):
                 self.args["--outdir"],
                 "network.txt",
                 "contig_data_network.txt",
-                temp_directory,
+                tmp_dir,
                 self.args["--threads"],
                 self.args["--normalization"],
                 False,
@@ -854,9 +933,10 @@ class Pipeline(AbstractCommand):
             network_file = self.args["--network"]
 
         # Partition the network
-        clustering_matrix, contigs_data_file = mtp.partition(
+        clustering_matrix_partition_file, contigs_data_file = mtp.partition(
             self.args["--algorithm"],
             fasta,
+            self.args["--cluster-matrix"],
             contigs_data_file,
             iterations,
             network_file,
@@ -865,7 +945,7 @@ class Pipeline(AbstractCommand):
             overlapping_parameter,
             resolution_parameter,
             size,
-            temp_directory,
+            tmp_dir,
             threads,
         )
 
@@ -878,9 +958,10 @@ class Pipeline(AbstractCommand):
 
         # Launch validation if desired.
         if not self.args["--skip-validation"]:
-            clustering_matrix_recursive = mtv.recursive_decontamination(
+            clustering_matrix_recursive_file = mtv.recursive_decontamination(
                 self.args["--algorithm"],
                 fasta,
+                self.args["--cluster-matrix"],
                 contigs_data_file,
                 final_fasta_dir,
                 overlapping_fasta_dir,
@@ -891,14 +972,25 @@ class Pipeline(AbstractCommand):
                 recursive_fasta_dir,
                 resolution_parameter,
                 size,
-                temp_directory,
+                tmp_dir,
                 threads,
             )
 
-            # Make the sum with the partiton clustering matrix
-            clustering_matrix = (
-                (clustering_matrix + clustering_matrix_recursive) / 2
-            ).tocoo()
+            if self.args["--cluster-matrix"]:
+                # Make the sum with the partiton clustering matrix and save it.
+                clustering_matrix = load_npz(
+                    clustering_matrix_partition_file + ".npz"
+                )
+                clustering_matrix_recursive = load_npz(
+                    clustering_matrix_recursive_file + ".npz"
+                )
+                clustering_matrix = (
+                    (clustering_matrix + clustering_matrix_recursive) / 2
+                ).tocoo()
+                clustering_matrix_file = join(
+                    self.args["--outdir"], "clustering_matrix"
+                )
+                save_npz(clustering_matrix_file, clustering_matrix)
 
             # Remove contig_data_partition file
             contig_data_partition_file = join(
@@ -906,12 +998,107 @@ class Pipeline(AbstractCommand):
             )
             os.remove(contig_data_partition_file)
 
-        # Save the clustering matrix
-        clustering_matrix_file = join(
-            self.args["--outdir"], "clustering_matrix.txt"
+        # Delete pyfastx index:
+        os.remove(fasta + ".fxi")
+        # Delete the temporary folder.
+        if not self.args["--no-clean-up"]:
+            shutil.rmtree(tmp_dir)
+
+
+class Contactmap(AbstractCommand):
+    """Generates a HiC contact map of a MetaTOR object from the pairs files, the
+    contig data file and a fasta file containing the contigs of interest.
+
+    Generates the Hi-C matrix of one MetaTOR object form the pair alignment file
+    of metaTOR. MetaTOR object integrated are contigs, and core, overlapping,
+    recursive, final or personalized bins. For the personalized bins you should
+    add a column in the contig_data_final.txt file with a header called "Other".
+
+    Do not display the matrix, you have to run another pipeline to display it
+    such hicstuff view or cooler show depending on the metrix format chosen.
+    It's also possible to try to scaffold the bin using instagraal pipeline
+    using the output.
+
+    usage:
+        contactmap --assembly=FILE --contig-data=FILE --enzyme=STR --name=STR --pairs=FILE
+        [--filter] [--force] [--mat-fmt=graal] [--min-size=5000]
+        [--no-clean-up] [--object=final_bin] [--outdir=DIR] [--pcr-dup]
+        [--tmpdir=DIR] [--threads=1]
+
+    options:
+        -a, --assembly=FILE     Path to the fasta file containing the contigs of
+                                interest. Could be the whole or the extracted
+                                contigs of one bin.
+        -c, --contig-data=FILE  Path to the contig_data_final.txt file form
+                                MetaTOR output.
+        -e, --enzyme=STR        The list of restriction enzyme used to digest
+                                the contigs separated by a comma. Example:
+                                HpaII,MluCI.
+        -n, --name=STR          Name of the metator or its numerical ID for the
+                                core bin, overlapping bin or recursive bin
+                                objects. Example: "NODE_1", "MetaTOR_1_0" or 8.
+        -p, --pairs=FILE        Path of the ".pairs" file. If more than one is
+                                given, files should be separated by a comma.
+        -D, --pcr-dup,          Filter out PCR duplicates based on read
+                                positions.
+        -f, --filter            Filter out spurious 3C events (loops and uncuts)
+                                using hicstuff filter. For more informations,
+                                see Cournac et al. BMC Genomics, 2012.
+        -F, --force             Write files even if the output files already
+                                exists.
+        -m, --mat-fmt=STR       The format of the output sparse matrix. Can be
+                                "bg2" for 2D Bedgraph format, "cool" for
+                                Mirnylab's cooler software, or "graal" for
+                                graal-compatible plain text COO format.
+                                [default: graal]
+        -N, --no-clean-up       If enabled intermediary files will be kept.
+        -o, --outdir=DIR        Output directory. Default creates a new
+                                directory contact_map in the current directory.
+        -O, --object=STR        Type of metator object used to build a contact
+                                map. Either "contig", "core_bin",
+                                "overlapping_bin", "recursive_bin", "final_bin"
+                                or "other". [Default: final_bin]
+        -s, --min-size=INT      Minimum size threshold to consider contigs.
+                                [Default: 5000]
+        -t, --threads=INT       Number of threads to allocate. [Default: 1]
+        -T, --tmpdir=DIR        Directory for storing intermediary files and
+                                temporary files. Default creates a "tmp" folder
+                                in the current directory.
+    """
+
+    def execute(self):
+
+        # Defined the temporary directory.
+        if not self.args["--tmpdir"]:
+            self.args["--tmpdir"] = "./tmp"
+        tmp_dir = mio.generate_temp_dir(self.args["--tmpdir"])
+
+        # Defined the output directory and output file names.
+        if not self.args["--outdir"]:
+            self.args["--outdir"] = join(
+                ".", "contact_map_" + self.args["--name"]
+            )
+        os.makedirs(self.args["--outdir"], exist_ok=True)
+
+        mtc.generate_contact_map(
+            self.args["--assembly"],
+            self.args["--contig-data"],
+            self.args["--enzyme"],
+            self.args["--name"],
+            self.args["--pairs"],
+            self.args["--outdir"],
+            tmp_dir,
+            self.args["--filter"],
+            self.args["--force"],
+            self.args["--mat-fmt"],
+            self.args["--object"],
+            int(self.args["--min-size"]),
+            self.args["--pcr-dup"],
+            int(self.args["--threads"]),
         )
-        save_npz(clustering_matrix_file, clustering_matrix)
 
         # Delete the temporary folder.
         if not self.args["--no-clean-up"]:
-            shutil.rmtree(temp_directory)
+            shutil.rmtree(tmp_dir)
+            # Delete pyfastx index:
+            os.remove(self.args["--assembly"] + ".fxi")
