@@ -72,11 +72,13 @@ def build_clustering_matrix(core_bins_contigs, hamming_distance, N):
     transition_matrix = sparse.coo_matrix(
         (values, (rows, cols)),
         shape=(len(core_bins_contigs), N + 1),
-        dtype=np.float32,
+        dtype=np.int32,
     )
-    # Compute the clustering matrix and keep only the upper triangle.
-    clustering_matrix = sparse.triu(
-        transition_matrix.T.dot(hamming_distance).dot(transition_matrix), k=1
+    # Compute the clustering matrix on only the upper triangle of the hamming
+    # distance matrix as it's symmetric to reduce memory usage.
+    hamming_distance = sparse.triu(hamming_distance, k=1)
+    clustering_matrix = transition_matrix.T.dot(hamming_distance).dot(
+        transition_matrix
     )
     return ((clustering_matrix + clustering_matrix) / 2).tocoo()
 
@@ -238,12 +240,16 @@ def generate_fasta(
         list_contigs_id = overlapping_bins[bin_id]
         list_contigs_name = []
         # Test if the bin is bigger than the size threshold given.
-        length_bin = contigs_data.Overlapping_bin_size[list_contigs_id[0] - 1]
+        length_bin = contigs_data.loc[
+            list_contigs_id[0] - 1, "Overlapping_bin_size"
+        ]
         if length_bin >= size:
             nb_bins += 1
             length_bins += length_bin
             for contig_id in list_contigs_id:
-                list_contigs_name.append(contigs_data.Name[contig_id - 1])
+                list_contigs_name.append(
+                    contigs_data.loc[contig_id - 1, "Name"]
+                )
             # Define the output file.
             output_file = join(output_dir, "MetaTOR_{0}_0.fa".format(bin_id))
             # Create the fasta file.
@@ -515,6 +521,7 @@ def louvain_iterations_cpp(network_file, iterations, tmp_dir, louvain_path):
 def partition(
     algorithm,
     assembly,
+    cluster_matrix,
     contig_data_file,
     iterations,
     network_file,
@@ -534,6 +541,8 @@ def partition(
         Algorithm to use to partition the network. Either leiden or louvain.
     assembly : str
         Path to the assembly file used for the partition.
+    cluster_matrix : bool
+        If True, build and save the clustering matrix.
     contig_data_file : str
         Path to the contig data table to update.
     iterations : int
@@ -640,13 +649,19 @@ def partition(
         temp_directory_bins,
     )
 
-    # Build clustering matrix.
-    logger.info("Build  clustering matrix")
-    clustering_matrix = build_clustering_matrix(
-        core_bins_contigs, hamming_distance, len(contigs_data.ID)
-    )
+    if cluster_matrix:
+        # Build clustering matrix and save it.
+        logger.info("Build  clustering matrix")
+        clustering_matrix = build_clustering_matrix(
+            core_bins_contigs, hamming_distance, len(contigs_data.ID)
+        )
+        # Save the clustering matrix
+        clustering_matrix_file = join(outdir, "clustering_matrix_partition")
+        sparse.save_npz(clustering_matrix_file, clustering_matrix)
+    else:
+        clustering_matrix_file = None
 
-    return clustering_matrix, contigs_data_file
+    return clustering_matrix_file, contigs_data_file
 
 
 def remove_isolates(output_partition, network_file):
@@ -735,9 +750,9 @@ def update_contigs_data(
         core_bin_contigs_number = len(core_bin)
         core_bin_length = sum(core_bin_data.Size)
         # Write the new information
-        contigs_data.Core_bin_ID[core_bin] = i + 1
-        contigs_data.Core_bin_contigs[core_bin] = core_bin_contigs_number
-        contigs_data.Core_bin_size[core_bin] = core_bin_length
+        contigs_data.loc[core_bin, "Core_bin_ID"] = i + 1
+        contigs_data.loc[core_bin, "Core_bin_contigs"] = core_bin_contigs_number
+        contigs_data.loc[core_bin, "Core_bin_size"] = core_bin_length
 
     # Add overlapping information
     for i in overlapping_bins:
@@ -747,12 +762,12 @@ def update_contigs_data(
         overlapping_bin_contigs_number = len(overlapping_bin)
         overlapping_bin_length = sum(overlapping_bin_data.Size)
         # Write the new information
-        contigs_data.Overlapping_bin_ID[overlapping_bin] = i
-        contigs_data.Overlapping_bin_contigs[
-            overlapping_bin
+        contigs_data.loc[overlapping_bin, "Overlapping_bin_ID"] = i
+        contigs_data.loc[
+            overlapping_bin, "Overlapping_bin_contigs"
         ] = overlapping_bin_contigs_number
-        contigs_data.Overlapping_bin_size[
-            overlapping_bin
+        contigs_data.loc[
+            overlapping_bin, "Overlapping_bin_size"
         ] = overlapping_bin_length
 
     # Write the new file
