@@ -20,11 +20,12 @@ Functions in this module:
     - recursive_clustering
     - recursive_decontamination
     - update_contigs_data_recursive
-    - write_bin_contigs
+    - write_bins_contigs
 """
 
 import logging
 import metator.io as mio
+import metator.figures as mtf
 import metator.partition as mtp
 import networkx as nx
 import numpy as np
@@ -207,36 +208,45 @@ def get_bin_coverage(bin_summary, contigs_data):
         the coverage.
     """
     # Compute HiC_coverage
+    total_hic_hit = 0
+    total_sg_hit = 0
     for i in range(len(contigs_data)):
-        bin_name = contigs_data.Final_bin[i]
+        bin_name = contigs_data.loc[i, "Final_bin"]
+        if contigs_data.loc[0, "Shotgun_coverage"] != "-":
+            total_sg_hit += (
+                contigs_data.loc[i, "Size"]
+                * contigs_data.loc[i, "Shotgun_coverage"]
+            )
+        total_hic_hit += contigs_data.loc[i, "Hit"]
         if bin_name != "ND":
             try:
-                bin_summary[bin_name]["HiC_Coverage"] += (
-                    1000
-                    * contigs_data.Hit[i]
-                    / int(bin_summary[bin_name]["size"])
+                bin_summary[bin_name]["HiC_abundance"] += (
+                    100 * contigs_data.loc[i, "Hit"]
                 )
             except KeyError:
-                bin_summary[bin_name]["HiC_Coverage"] = (
-                    1000
-                    * contigs_data.Hit[i]
-                    / int(bin_summary[bin_name]["size"])
+                bin_summary[bin_name]["HiC_abundance"] = (
+                    100 * contigs_data.loc[i, "Hit"]
                 )
 
             # If no depth files were given do not compute the Shotgun coverage.
-            if contigs_data.Shotgun_coverage[0] != "-":
+            if contigs_data.loc[0, "Shotgun_coverage"] != "-":
                 try:
-                    bin_summary[bin_name]["SG_Coverage"] += (
-                        contigs_data.Size[i]
-                        * contigs_data.Shotgun_coverage[i]
-                        / int(bin_summary[bin_name]["size"])
+                    bin_summary[bin_name]["SG_abundance"] += (
+                        contigs_data.loc[i, "Size"]
+                        * contigs_data.loc[i, "Shotgun_coverage"]
+                        * 100
                     )
                 except KeyError:
-                    bin_summary[bin_name]["SG_Coverage"] = (
-                        contigs_data.Size[i]
-                        * contigs_data.Shotgun_coverage[i]
-                        / int(bin_summary[bin_name]["size"])
+                    bin_summary[bin_name]["SG_abundance"] = (
+                        contigs_data.loc[i, "Size"]
+                        * contigs_data.loc[i, "Shotgun_coverage"]
+                        * 100
                     )
+    for bin_name in bin_summary:
+        # Divide the HiC abundance by two as the hit are counted twice.
+        bin_summary[bin_name]["HiC_abundance"] /= 2 * total_hic_hit
+        if contigs_data.loc[0, "Shotgun_coverage"] != "-":
+            bin_summary[bin_name]["SG_abundance"] /= total_sg_hit
     return bin_summary
 
 
@@ -323,6 +333,7 @@ def recursive_clustering(
     taxonomy_file,
     contigs_data_file,
     network_file,
+    cluster_matrix,
     size,
     threads,
 ):
@@ -356,6 +367,8 @@ def recursive_clustering(
         Path to the contigs data file from metator partition.
     network_file : str
         Path to the network file from metator network.
+    cluster_matrix : bool
+        If True, build the clustering matrix and save it.
     size : int
         Size threshodl in base pairs of the bins.
     threads : int
@@ -421,11 +434,8 @@ def recursive_clustering(
             bin_id = str(bin_id.split("_")[1])
 
             # Extract contigs
-            list_contigs = list(
-                contigs_data["ID"][
-                    contigs_data.Overlapping_bin_ID.apply(str) == bin_id
-                ]
-            )
+            mask = contigs_data["Overlapping_bin_ID"].apply(str) == bin_id
+            list_contigs = list(contigs_data.loc[mask, "ID"])
 
             # Extract subnetwork
             subnetwork = network.subgraph(list_contigs)
@@ -492,19 +502,29 @@ def recursive_clustering(
                 contamination,
             )
 
-            clustering_matrix += mtp.build_clustering_matrix(
-                recursive_core_bins, hamming_distance, N
-            )
+            # Build the clustering matrix of the subnetwork and add it.
+            if cluster_matrix:
+                clustering_matrix += mtp.build_clustering_matrix(
+                    recursive_core_bins, hamming_distance, N
+                )
 
             # Put back the info log
             logger.setLevel(logging.INFO)
 
-    return contamination, contigs_data, clustering_matrix
+    # Save the clustering matrix
+    if cluster_matrix:
+        clustering_matrix_file = join(outdir, "clustering_matrix_recursive")
+        sparse.save_npz(clustering_matrix_file, clustering_matrix)
+    else:
+        clustering_matrix_file = None
+
+    return contamination, contigs_data, clustering_matrix_file
 
 
 def recursive_decontamination(
     algorithm,
     assembly,
+    cluster_matrix,
     contig_data_file,
     final_fasta_dir,
     input_fasta_dir,
@@ -528,6 +548,8 @@ def recursive_decontamination(
         louvain.
     assembly : str
         Path to the assembly file used for the partition.
+    cluster_matrix : bool
+        If True, build the clustering matrix and save it.
     contig_data_file : str
         Path to the contig data table to update.
     final_fasta_dir : str
@@ -586,7 +608,7 @@ def recursive_decontamination(
     )
 
     # Iterates Louvain or Leiden on contaminated and complete bins.
-    contamination, contigs_data, clustering_matrix = recursive_clustering(
+    contamination, contigs_data, clustering_matrix_file = recursive_clustering(
         assembly,
         iterations,
         overlapping_parameter,
@@ -599,6 +621,7 @@ def recursive_decontamination(
         overlapping_taxonomy_file,
         contig_data_file,
         network_file,
+        cluster_matrix,
         size,
         threads,
     )
@@ -657,10 +680,15 @@ def recursive_decontamination(
     mio.write_checkm_summary(bin_summary, bin_summary_file)
 
     # Write the new file
-    contig_data_file_2 = join(outdir, "contig_data_final.txt")
-    contigs_data.to_csv(contig_data_file_2, sep="\t", header=True, index=False)
+    contig_data_file_final = join(outdir, "contig_data_final.txt")
+    contigs_data.to_csv(
+        contig_data_file_final, sep="\t", header=True, index=False
+    )
 
-    return clustering_matrix
+    # Plot some figures of contigs distribution inside bins:
+    mtf.plot_figures(outdir, contigs_data, bin_summary, size)
+
+    return clustering_matrix_file
 
 
 def update_contigs_data_recursive(
@@ -707,43 +735,43 @@ def update_contigs_data_recursive(
         recursive_bin_contigs_number = len(recursive_bin)
         recursive_bin_length = sum(recursive_bin_data.Size)
 
-        if recursive_bin_contigs_number > 1:
+        if recursive_bin_length > size:
+            # If one new bin is generated change the boolean value to True
+            contamination = True
+
             # Write the new information
-            contigs_data.Recursive_bin_ID[recursive_bin] = rec_id
-            contigs_data.Recursive_bin_contigs[
-                recursive_bin
+            contigs_data.loc[recursive_bin, "Recursive_bin_ID"] = rec_id
+            contigs_data.loc[
+                recursive_bin, "Recursive_bin_contigs"
             ] = recursive_bin_contigs_number
-            contigs_data.Recursive_bin_size[
-                recursive_bin
+            contigs_data.loc[
+                recursive_bin, "Recursive_bin_size"
             ] = recursive_bin_length
 
-            if recursive_bin_length > size:
+            # Defined name of the recursive bin
+            oc_id = contigs_data.loc[recursive_bin[0], "Overlapping_bin_ID"]
+            output_file = join(
+                outdir, "MetaTOR_{0}_{1}.fa".format(oc_id, rec_id)
+            )
 
-                # If one new bin is generated change the boolean value to True
-                contamination = True
+            # Retrieve names of the contigs
+            list_contigs = list(contigs_data.loc[recursive_bin, "Name"])
 
-                # Defined name of the recursive bin
-                oc_id = contigs_data.Overlapping_bin_ID[recursive_bin[0]]
-                output_file = join(
-                    outdir, "MetaTOR_{0}_{1}.fa".format(oc_id, rec_id)
-                )
+            # Generate the fasta
+            contigs_file = join(
+                tmpdir, "MetaTOR_{0}_{1}.txt".format(oc_id, rec_id)
+            )
+            with open(contigs_file, "w") as f:
+                for contig in list_contigs:
+                    f.write("{0}\n".format(contig))
+            cmd = "pyfastx extract {0} -l {1} > {2}".format(
+                assembly, contigs_file, output_file
+            )
+            process = sp.Popen(cmd, shell=True)
+            process.communicate()
 
-                # Retrieve names of the contigs
-                list_contigs = list(contigs_data.Name[recursive_bin])
-
-                # Generate the fasta
-                contigs_file = join(
-                    tmpdir, "MetaTOR_{0}_{1}.txt".format(oc_id, rec_id)
-                )
-                with open(contigs_file, "w") as f:
-                    for contig in list_contigs:
-                        f.write("{0}\n".format(contig))
-                cmd = "pyfastx extract {0} -l {1} > {2}".format(
-                    assembly, contigs_file, output_file
-                )
-                process = sp.Popen(cmd, shell=True)
-                process.communicate()
-                rec_id += 1
+            # Add one to the recursive id
+            rec_id += 1
 
     return contamination, contigs_data
 
@@ -780,8 +808,8 @@ def write_bins_contigs(bin_summary, contigs_data, outfile):
     # Write the contigs id with their bins id in table file
     with open(outfile, "w") as f:
         for i in range(len(contigs_data)):
-            over_id = str(contigs_data.Overlapping_bin_ID[i])
-            rec_id = str(contigs_data.Recursive_bin_ID[i])
+            over_id = str(contigs_data.loc[i, "Overlapping_bin_ID"])
+            rec_id = str(contigs_data.loc[i, "Recursive_bin_ID"])
             try:
                 rec_ids = list_bin_id[over_id]
                 binned = False
@@ -798,9 +826,11 @@ def write_bins_contigs(bin_summary, contigs_data, outfile):
                         over_id,
                         rec_id,
                     )
-                    contigs_data.Final_bin[i] = final_bin
+                    contigs_data.loc[i, "Final_bin"] = final_bin
                     f.write(
-                        "{0}\t{1}\n".format(contigs_data.Name[i], final_bin)
+                        "{0}\t{1}\n".format(
+                            contigs_data.loc[i, "Name"], final_bin
+                        )
                     )
             except KeyError:
                 pass
